
#include "ip4addr.h"

struct zone;
struct zonetype;
struct zonedata;
struct dnspacket;

void zlog(int level, int lineno, const char *fmt, ...);
void zwarn(int lineno, const char *fmt, ...);
void zloaded(const char *fmt, ...);

#define R_A_DEFAULT ((ip4addr_t)0x7f000002)

typedef struct zonedata *z_allocfn();
typedef int z_loadfn(struct zonedata *z, FILE *f);
typedef int z_finishfn(struct zonedata *z);
typedef void z_freefn(struct zonedata *zone);
typedef int
z_queryfn(const struct zonedata *const zone, struct dnspacket *p,
          const unsigned char *const query, unsigned qtyp);

/* flags used in qtyp */
#define NSQUERY_TXT	(1u<< 8)
#define NSQUERY_A	(1u<< 9)
#define NSQUERY_NS	(1u<<10)
#define NSQUERY_SOA	(1u<<11)

#define NSQUERY_A_TXT	(NSQUERY_A|NSQUERY_TXT)
#define NSQUERY_ANY	0xff00u

struct zonetype {
  const char *name;     /* name of the type */
  unsigned qfilter;	/* which records we recognize */
  z_queryfn *queryfn;   /* routine to perform query */
  z_allocfn *allocfn;	/* allocation routine */
  z_loadfn *loadfn;     /* routine to load zone data */
  z_finishfn *finishfn;	/* finish loading */
  z_freefn *freefn;     /* routine to free zone data */
  const char *descr;    /* short description of a zone type */
};

#define declarezonetype(t) extern const struct zonetype t##_zone
#define definezonetype(t, qfilter, descr) \
 static z_allocfn t##_alloc; \
 static z_queryfn t##_query; \
 static z_loadfn t##_load; \
 static z_finishfn t##_finish; \
 static z_freefn t##_free; \
 const struct zonetype t##_zone = { \
   #t, /* name */ qfilter, \
   t##_query, t##_alloc, t##_load, t##_finish, t##_free, \
   descr }

declarezonetype(ip4set);
declarezonetype(ip4vset);
declarezonetype(dnset);
declarezonetype(dnvset);
declarezonetype(generic);

int
readzlines(FILE *f, struct zonedata *z,
           int (*zlpfn)(struct zonedata *z,
                        char *line, int lineno, int llines));

/* see details of DNS packet structure in rbldnsd.c */

int addrec_a(struct dnspacket *p, ip4addr_t aip);
int addrec_txt(struct dnspacket *p, const char *txt, const char *subst);
int addrec_any(struct dnspacket *p, unsigned dtp,
               const void *data, unsigned dsz);

ip4addr_t dntoip4addr(const unsigned char *q);

int addrtxt(char *str, ip4addr_t *ap, char **txtp);

void oom();
void *emalloc(unsigned size);
void *erealloc(void *ptr, unsigned size);
char *estrdup(const char *str);
struct mempool;
void *mp_ealloc(struct mempool *mp, unsigned size);
char *mp_estrdup(struct mempool *mp, const char *str);
const char *mp_edstrdup(struct mempool *mp, const char *str);
